const createDispatch = gates => action => {
    if (action.__type === 'batch') {
        return processBatch(gates, action);

    } else {
        return processAction(gates, action)
    }
}

const processAction = (gates, action) => gates[action.gate](action);

const processBatch = (gates, batch) =>  ({
    payload: Object.assign({}, ...Object.keys(batch.gateBatches).map(gateName =>
        gates[gateName](batch.gateBatches[gateName])
    )),
    errors: []
})

const applyMiddleware = (dispatch, gates, middlewares) => {
    middlewares = middlewares.slice();
    middlewares.reverse();

    let prevDispatch = dispatch;
    middlewares.forEach(middleware => prevDispatch = middleware(gates)(prevDispatch));
    return prevDispatch;
};

const createWatergate = (gates, middleware) => ({
    dispatch: middleware ? applyMiddleware(createDispatch(gates), gates, middleware) : createDispatch(gates)
});

export default createWatergate;
